@@ -8,7 +8,7 @@ package Event::Distributor::Signal;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use Future;
 
@@ -24,8 +24,17 @@ sub fire
    my ( $dist, $args, $subscribers ) = @_;
 
    return Future->wait_all(
-      map { $_->( $dist, @$args ) } @$subscribers
-   )->then_done( () );
+      map { Future->call( $_, $dist, @$args ) } @$subscribers
+   )->then( sub {
+      my @failed = grep { $_->failure } @_;
+
+      return Future->done() if !@failed;
+      return $failed[0] if @failed == 1;
+      return Future->fail( "Multiple subscribers failed:\n" .
+         join( "", map { " | " . $_->failure } @failed ),
+         distributor => @failed,
+      );
+   });
 }
 
 0x55AA
